// BASS 2.3 add-on C/C++ header file, copyright (c) 2003-2006 Ian Luck.

#include "bass.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef void *BASSFILE;

typedef struct {
	void (WINAPI *Free)(HSTREAM handle);
	QWORD (WINAPI *GetLength)(HSTREAM handle);
	const char *(WINAPI *GetTags)(HSTREAM handle, DWORD tags);
	void (WINAPI *PreBuf)(HSTREAM handle, DWORD len);
	DWORD (WINAPI *GetFilePosition)(HSTREAM handle, DWORD mode); // optional (can be NULL) when using BASSFILE routines
	void (WINAPI *GetInfo)(HSTREAM handle, BASS_CHANNELINFO *info);
	BOOL (WINAPI *CanSetPosition)(HSTREAM handle, QWORD pos);
	QWORD (WINAPI *SetPosition)(HSTREAM handle, QWORD pos);
	QWORD (WINAPI *GetPosition)(HSTREAM handle, QWORD pos); // optional (can be NULL)
	HSYNC (WINAPI *SetSync)(HSTREAM handle, DWORD type, QWORD param, SYNCPROC *proc, DWORD user); // not required if not setting syncs
	void (WINAPI *RemoveSync)(HSTREAM handle, HSYNC sync); // not required if not setting syncs
	BOOL (WINAPI *CanResume)(HSTREAM handle); // optional (can be NULL)
	DWORD (WINAPI *SetFlags)(HSTREAM handle, DWORD flags);
} ADDON_FUNCTIONS;

typedef HSTREAM (CALLBACK STREAMCREATEPROC)(BASSFILE file, DWORD flags); // BASS_StreamCreateFile/User/URL plugin function
typedef HSTREAM (CALLBACK STREAMCREATEURLPROC)(const char *url, DWORD offset, DWORD flags, DOWNLOADPROC *proc, DWORD user); // BASS_StreamCreateURL plugin function (unsupported protocol)
typedef DWORD (CALLBACK BASSCONFIGPROC)(DWORD option, DWORD value, BOOL set); // BASS_Set/GetConfig plugin function

typedef struct {
	void (WINAPI *SetError)(int error); // set error code
	HSTREAM (WINAPI *CreateStream)(DWORD freq, DWORD chan, DWORD flags, STREAMPROC *proc, DWORD user, const ADDON_FUNCTIONS *funcs); // create a stream
	HSYNC (WINAPI *NewSync)(HSTREAM handle, DWORD type, SYNCPROC *proc, DWORD user); // add a sync to a channel
	BOOL (WINAPI *TriggerSync)(HSTREAM handle, HSYNC sync, QWORD pos, DWORD data); // trigger a sync
	QWORD (WINAPI *GetCount)(HSTREAM handle, int pos); // get raw written bytes count (pos=0), or translated to position (pos=1), or translated delta to position (pos<0)

// --- 2.2 stuff ---

	void (WINAPI *RegisterPluginConfig)(BASSCONFIGPROC *proc, BOOL remove); // add/remove a BASS_Set/GetConfig plugin

	struct { // file functions
		BASSFILE (WINAPI *Open)(BOOL mem, const void *file, DWORD offset, DWORD length, DWORD flags, BOOL readtags); // open a file
		BASSFILE (WINAPI *OpenURL)(const char *url, DWORD offset, DWORD flags, DOWNLOADPROC *proc, DWORD user, BOOL readtags); // open a URL
		BASSFILE (WINAPI *OpenUser)(BOOL buffered, DWORD flags, STREAMFILEPROC *proc, DWORD user, BOOL readtags); // open a custom file
		void (WINAPI *Close)(BASSFILE file); // close an opened file
		const void *(WINAPI *GetFileName)(BASSFILE file, BOOL *unicode); // get the filename/url (NULL=memory/custom file)
		BOOL (WINAPI *SetStream)(BASSFILE file, HSTREAM handle); // set stream handle (for auto closing and META/DOWNLOAD syncs)
		DWORD (WINAPI *GetFlags)(BASSFILE file); // get BASSFILE_xxx flags
		void (WINAPI *SetFlags)(BASSFILE file, DWORD flags); // set BASSFILE_xxx flags (BASSFILE_RESTRICT only)
		DWORD (WINAPI *Read)(BASSFILE file, void *buf, DWORD len); // read from file
		BOOL (WINAPI *Seek)(BASSFILE file, DWORD pos); // seek in file
		DWORD (WINAPI *GetPos)(BASSFILE file, DWORD mode); // get file position (mode=BASS_FILEPOS_xxx)
		BOOL (WINAPI *Eof)(BASSFILE file); // End of file?
		const char *(WINAPI *GetTags)(BASSFILE file, DWORD tags); // get tags (ID3/ID3V2/HTTP/ICY/META)
		// net/buffered-only stuff
		BOOL (WINAPI *StartThread)(BASSFILE f, DWORD bitrate, DWORD offset); // start download thread (bitrate in bytes/sec)
		BOOL (WINAPI *IsConnected)(BASSFILE file); // connection is still up?
		BOOL (WINAPI *CanResume)(BASSFILE file); // enough data buffered to resume?
	} file;

// --- 2.3 stuff ---

	struct { // sample data processing functions (len=samples, dst can = src)
		void (WINAPI *float2int)(const float *src, void *dst, DWORD len, DWORD res); // convert floating-point data to 8 or 16-bit (res=1 or 2)
		void (WINAPI *int2float)(const void *src, float *dst, DWORD len, DWORD res); // convert 8/16/24/32-bit (res=1/2/3/4) to floating-point
		void (WINAPI *swap)(BYTE *src, DWORD len, DWORD res); // swap byte order of 16 or 32-bit (res=2 or 4) data
	} data;

} BASS_FUNCTIONS;

#define BASSFILE_BUFFERED	1	// net/buffered file
#define BASSFILE_BLOCK		BASS_STREAM_BLOCK
#define BASSFILE_RESTRICT	BASS_STREAM_RESTRICT

typedef const void *(WINAPI BASSPLUGIN)(DWORD face); // plugin interface function

// BASSplugin "faces"
#define BASSPLUGIN_INFO			0 // BASS_PLUGININFO
#define BASSPLUGIN_CREATE		1 // STREAMCREATEPROC
#define BASSPLUGIN_CREATEURL	2 // STREAMCREATEURLPROC

#ifndef bassfunc
#ifdef _WIN32
extern __declspec(dllimport) const BASS_FUNCTIONS _;
#else
extern const BASS_FUNCTIONS _;
#endif
#define bassfunc (&_)
#endif

#ifdef __cplusplus
}
#endif
